﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using IdeaBlade.EntityModel;
using IdeaBlade.Validation;
using DevForceSilverlightApp;
using System.Collections.ObjectModel;
using System.Windows.Navigation;
using System.Collections;
using System.Text;
using IdeaBlade.EntityModel.Security;
#endregion Using Statements

namespace DevForceSilverlightApp {
  public partial class MainPage : UserControl {

    #region ctor and startup
    public MainPage() {
      InitializeComponent();
      Loaded += Page_Loaded;
    }

    void Page_Loaded(object sender, RoutedEventArgs e) {
      CreateEntityManager();
      PreloadVerifiers(_mgr.VerifierEngine);
      ConfigureDataForm();
      ConfigureDataGrid();
      Login();
    }

    private void PreloadVerifiers(VerifierEngine aVerifierEngine) {
      aVerifierEngine.DiscoverVerifiers(typeof(Order));
    }
    #endregion ctor and startup

    #region Login
    private async void Login() {
      var cred = new LoginCredential("demo", "demo", "demo");
      WriteMessage("Logging in ...");
      try {
        _mgr.AuthenticationContext = await Authenticator.Instance.LoginAsync(cred);
        //InitializeItemsSources();
        LoadData();
      } catch (Exception e) {
        WriteMessage(e.Message);
      }
    }

    #endregion Login

    #region Load Data
    private void CreateEntityManager() {
      WriteMessage("Creating EntityManager ...");
      _mgr = new DomainModelEntityManager(true);
    }

    private async void LoadData() {
      try {
        var results = await _mgr.ExecuteQueryAsync(_mgr.Employees);//, GotEmployees, null);
        foreach (Employee aEmployee in results) {
          _employees.Add(aEmployee);
        }
        // Set CurrentItem to first Employee so form will flesh out
        if (_employees.Count > 0) {
          _employeeDataForm.CurrentItem = _employees[0];
        }
        ReportFetchCount(results, "Employee");
      } catch (Exception e) {
        WriteMessage(e.Message);
      }
    }

    void Orders_PendingEntityListResolved(object sender, PendingEntityListResolvedEventArgs<Order> e) {
      PopulateOrderList(e.ResolvedEntities);
      ReportFetchCount(e.ResolvedEntities, "Order");
      ((RelatedEntityList<Order>)sender).PendingEntityListResolved -= Orders_PendingEntityListResolved;
    }
    #endregion Load Data

    #region Configure Environment
    private void ConfigureDataForm() {
      _employeeDataForm.CommandButtonsVisibility = DataFormCommandButtonsVisibility.Navigation;
      _employeeDataForm.ItemsSource = _employees;
      _employeeDataForm.CurrentItemChanged += new EventHandler<EventArgs>(_employeeDataForm_CurrentItemChanged);
    }

    void _employeeDataForm_CurrentItemChanged(object sender, EventArgs e) {
      Employee currentEmployee = (Employee)_employeeDataForm.CurrentItem;
      WriteMessage("Now viewing Employee " + (currentEmployee).LastName + ", " + (currentEmployee).FirstName);
      _orders.Clear();
      if (currentEmployee.Orders.IsPendingEntityList) {
        currentEmployee.Orders.PendingEntityListResolved +=
          new EventHandler<PendingEntityListResolvedEventArgs<Order>>(Orders_PendingEntityListResolved);
      }
      else {
        PopulateOrderList(currentEmployee.Orders);
      }
    }

    private void PopulateOrderList(IEnumerable<Order> orders) {
      foreach (Order anOrder in orders) {
        _orders.Add(anOrder);
      }
    }

    private void ConfigureDataGrid() {
      this._ordersDataGrid.ItemsSource = _orders;
    }
    #endregion Configure Environment

    #region Methods to Display Status
    private void ReportFetchCount(IEnumerable result, string entityType) {
      int entitiesRetrieved = ((ICollection)result).Count;
      WriteMessage(string.Format("Retrieved {0} entities of type " + entityType, entitiesRetrieved));
    }

    private void WriteMessage(string msg) {
      _msgNumber += 1;
      _statusStringBuilder.Append(String.Format("{0}", _msgNumber.ToString("D4")) +
        " " + msg + Environment.NewLine);
      _statusTextBlock.Text = _statusStringBuilder.ToString();
      _statusMsg_ScrollViewer.ScrollToVerticalOffset(_statusMsg_ScrollViewer.ScrollableHeight);
    }
    #endregion Methods to Display Status

    #region Private Fields
    ObservableCollection<Employee> _employees = new ObservableCollection<Employee>();
    ObservableCollection<Order> _orders = new ObservableCollection<Order>();
    DomainModelEntityManager _mgr;
    int _msgNumber = 0;
    StringBuilder _statusStringBuilder = new StringBuilder();
    #endregion Private Fields
  }
}
