﻿using System;
using System.Linq;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;
using IdeaBlade.Validation;
using System.Collections.Generic;

namespace DevForceSilverlightApp {
  
  public partial class Order {

    #region Verification
    /// <summary>Get Verifiers.</summary>
    /// <param name="pVerifierProviderContext">Context in which these Verifiers are retrieved.</param>
    /// <returns>The verifiers.</returns>
    public class VerifierProvider : IVerifierProvider {

      public IEnumerable<Verifier> GetVerifiers(
        Object pVerifierProviderContext) {

        List<Verifier> verifiers = new List<Verifier>();

        verifiers.Add(GetOrderDateAfterHiredVerifier());

        return verifiers;
      }

      ///<summary>Get the OrderDateAfterHired Verifier.</summary>
      private static Verifier GetOrderDateAfterHiredVerifier() {
        string description =
          "OrderDate must be after the sales rep's HireDate.";
        DelegateVerifier<Order> v =
          new DelegateVerifier<Order>(
            description, OrderDateAfterHiredCondition);
        v.VerifierOptions.ExecutionModes =
          VerifierExecutionModes.InstanceAndOnAfterSetTriggers;
        v.AddTrigger(Order.PathFor(o => o.OrderDate));

        TriggerItem aTriggerItem = new TriggerItem(typeof(Employee), Employee.PathFor(e => e.HireDate));
        // TriggerLink has one fewer signature in Silverlight than in WPF, due to the
        // non-support of PropertyDescriptors in Silverlight. This example uses a different
        // constructor than the one used in the WPF example.

        TriggerLink aTriggerLink = new TriggerLink(aTriggerItem, GetOrdersAssociatedWithEmployee, true);
        v.AddTrigger(aTriggerLink);

        return v;
      }

      public static IEnumerable<Order> GetOrdersAssociatedWithEmployee(Object employee) {
        var anEmployee = (Employee)employee;
        return anEmployee.Orders;
      }


      ///<summary>
      ///Implementor of the OrderDateAfterHired Verifier condition.
      ///</summary>
      private static VerifierResult OrderDateAfterHiredCondition(
        Order pTarget, TriggerContext pTriggerContext,
        VerifierContext pVerifierContext) {

        if (pTriggerContext != null &&
          pTriggerContext.Timing == TriggerTiming.BeforeSet) {
          throw new VerifierException(
        "OrderDateAfterHired verifier not implemented for BeforeSet");
        }
        bool isOk =
          pTarget.OrderDate.HasValue &&
            pTarget.SalesRep.HireDate.HasValue &&
          pTarget.OrderDate.Value >=
            pTarget.SalesRep.HireDate.Value;

        return new VerifierResult(isOk);
      }

    #endregion
    }

  }

}
