﻿#Region "Using Statements"
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes

Imports IdeaBlade.EntityModel
Imports IdeaBlade.Validation
Imports DevForceSilverlightApp
Imports System.Collections.ObjectModel
Imports System.Windows.Navigation
Imports System.Collections
Imports System.Text
Imports IdeaBlade.EntityModel.Security

#End Region ' Using Statements

Namespace DevForceSilverlightApp
  Partial Public Class MainPage
    Inherits UserControl

#Region "ctor and startup"
    Public Sub New()
      InitializeComponent()
      AddHandler Loaded, AddressOf Page_Loaded
    End Sub

    Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
      CreateEntityManager()
      PreloadVerifiers(_mgr.VerifierEngine)
      ConfigureDataForm()
      ConfigureDataGrid()
      Login()
    End Sub

    Private Sub PreloadVerifiers(ByVal aVerifierEngine As VerifierEngine)
      aVerifierEngine.DiscoverVerifiers(GetType(Order))
    End Sub
#End Region ' ctor and startup

#Region "Login"
    Private Async Sub Login()
      Dim cred = New LoginCredential("demo", "demo", "demo")
      WriteMessage("Logging in ...")
      Try
        _mgr.AuthenticationContext = Await Authenticator.Instance.LoginAsync(cred)
        WriteMessage("Logged in")
        'InitializeItemsSources();
        LoadData()
      Catch ex As Exception
        WriteMessage(ex.Message)
      End Try
    End Sub
#End Region ' Login

#Region "Load Data"
    Private Sub CreateEntityManager()
      WriteMessage("Creating EntityManager ...")
      _mgr = New DomainModelEntityManager(True)
    End Sub

    Private Async Sub LoadData()
      Try
        Dim results = Await _mgr.ExecuteQueryAsync(_mgr.Employees)

        For Each aEmployee As Employee In results
          _employees.Add(aEmployee)
        Next aEmployee
        ' Set CurrentItem to first Employee so form will flesh out
        If _employees.Count > 0 Then
          _employeeDataForm.CurrentItem = _employees(0)
        End If
        ReportFetchCount(results, "Employee")
      Catch ex As Exception
        WriteMessage(ex.Message)
      End Try

    End Sub

    Private Sub Orders_PendingEntityListResolved(ByVal sender As Object, ByVal e As PendingEntityListResolvedEventArgs(Of Order))
      PopulateOrderList(e.ResolvedEntities)
      ReportFetchCount(e.ResolvedEntities, "Order")
      RemoveHandler (CType(sender, RelatedEntityList(Of Order))).PendingEntityListResolved, AddressOf Orders_PendingEntityListResolved
    End Sub
#End Region ' Load Data

#Region "Configure Environment"
    Private Sub ConfigureDataForm()
      _employeeDataForm.CommandButtonsVisibility = DataFormCommandButtonsVisibility.Navigation
      _employeeDataForm.ItemsSource = _employees
      AddHandler _employeeDataForm.CurrentItemChanged, AddressOf _employeeDataForm_CurrentItemChanged
    End Sub

    Private Sub _employeeDataForm_CurrentItemChanged(ByVal sender As Object, ByVal e As EventArgs)
      Dim currentEmployee As Employee = CType(_employeeDataForm.CurrentItem, Employee)
      WriteMessage("Now viewing Employee " & (currentEmployee).LastName & ", " & (currentEmployee).FirstName)
      _orders.Clear()
      If currentEmployee.Orders.IsPendingEntityList Then
        AddHandler currentEmployee.Orders.PendingEntityListResolved, AddressOf Orders_PendingEntityListResolved
      Else
        PopulateOrderList(currentEmployee.Orders)
      End If
    End Sub

    Private Sub PopulateOrderList(ByVal orders As IEnumerable(Of Order))
      For Each anOrder As Order In orders
        _orders.Add(anOrder)
      Next anOrder
    End Sub

    Private Sub ConfigureDataGrid()
      Me._ordersDataGrid.ItemsSource = _orders
    End Sub
#End Region ' Configure Environment

#Region "Methods to Display Status"
    Private Sub ReportFetchCount(ByVal result As IEnumerable, ByVal entityType As String)
      Dim entitiesRetrieved As Integer = (CType(result, ICollection)).Count
      WriteMessage(String.Format("Retrieved {0} entities of type " & entityType, entitiesRetrieved))
    End Sub

    Private Sub WriteMessage(ByVal msg As String)
      _msgNumber += 1
      _statusStringBuilder.Append(String.Format("{0}", _msgNumber.ToString("D4")) & " " & msg & Environment.NewLine)
      _statusTextBlock.Text = _statusStringBuilder.ToString()
      _statusMsg_ScrollViewer.ScrollToVerticalOffset(_statusMsg_ScrollViewer.ScrollableHeight)
    End Sub
#End Region ' Methods to Display Status

#Region "Private Fields"
    Private _employees As New ObservableCollection(Of Employee)()
    Private _orders As New ObservableCollection(Of Order)()
    Private _mgr As DomainModelEntityManager
    Private _msgNumber As Integer = 0
    Private _statusStringBuilder As New StringBuilder()
#End Region ' Private Fields
  End Class
End Namespace
