﻿Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core
Imports IdeaBlade.Validation

Partial Public Class Order
    Implements IVerifierProvider

#Region "Verification"
    ''' <summary>Get Verifiers.</summary>
    ''' <param name="pVerifierProviderContext">Context in which these Verifiers are retrieved.</param>
    ''' <returns>The verifiers.</returns>

    Public Function GetVerifiers(ByVal pVerifierProviderContext As Object) As IEnumerable(Of Verifier) Implements IVerifierProvider.GetVerifiers

        Dim verifiers As New List(Of Verifier)()

        verifiers.Add(GetOrderDateAfterHiredVerifier())

        Return verifiers
    End Function

    '''<summary>Get the OrderDateAfterHired Verifier.</summary>
    Private Shared Function GetOrderDateAfterHiredVerifier() As Verifier
        Dim description As String = "OrderDate must be after the sales rep's HireDate."
        Dim v As New DelegateVerifier(Of Order)(description, AddressOf OrderDateAfterHiredCondition)
        v.VerifierOptions.ExecutionModes = VerifierExecutionModes.InstanceAndOnAfterSetTriggers
        v.AddTrigger(Order.PathFor(Function(o) o.OrderDate))

        Dim aTriggerItem As New TriggerItem(GetType(Employee), Employee.PathFor(Function(e) e.HireDate))
        ' TriggerLink has one fewer signature in Silverlight than in WPF, due to the
        ' non-support of PropertyDescriptors in Silverlight. This example uses a different
        ' constructor than the one used in the WPF example.

        Dim aTriggerLink As New TriggerLink(aTriggerItem, AddressOf GetOrdersAssociatedWithEmployee, True)
        v.AddTrigger(aTriggerLink)

        Return v
    End Function

    Public Shared Function GetOrdersAssociatedWithEmployee(ByVal employee As Object) As IEnumerable(Of Order)
        Dim anEmployee = CType(employee, Employee)
        Return anEmployee.Orders
    End Function


    '''<summary>
    '''Implementor of the OrderDateAfterHired Verifier condition.
    '''</summary>
    Private Shared Function OrderDateAfterHiredCondition(ByVal pTarget As Order, ByVal pTriggerContext As TriggerContext, ByVal pVerifierContext As VerifierContext) As VerifierResult

        If pTriggerContext IsNot Nothing AndAlso pTriggerContext.Timing = TriggerTiming.BeforeSet Then
            Throw New VerifierException("OrderDateAfterHired verifier not implemented for BeforeSet")
        End If
        Dim isOk As Boolean = pTarget.OrderDate.HasValue AndAlso pTarget.SalesRep.HireDate.HasValue AndAlso pTarget.OrderDate.Value >= pTarget.SalesRep.HireDate.Value

        Return New VerifierResult(isOk)
    End Function

#End Region

End Class
